@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -73,20 +73,27 @@ NOEXPORT unsigned psk_server_callback(SSL *, const char *,
 #endif /* !defined(OPENSSL_NO_PSK) */
 NOEXPORT int load_cert_file(SERVICE_OPTIONS *);
 NOEXPORT int load_key_file(SERVICE_OPTIONS *);
+NOEXPORT int pkcs12_extension(const char *);
+NOEXPORT int load_pkcs12_file(SERVICE_OPTIONS *);
 #ifndef OPENSSL_NO_ENGINE
 NOEXPORT int load_cert_engine(SERVICE_OPTIONS *);
 NOEXPORT int load_key_engine(SERVICE_OPTIONS *);
 #endif
-NOEXPORT int password_cb(char *, int, int, void *);
+NOEXPORT int passphrase_cb(char *, int, int, void *);
+NOEXPORT int ui_retry();
 
 /* session callbacks */
 NOEXPORT int sess_new_cb(SSL *, SSL_SESSION *);
-NOEXPORT SSL_SESSION *sess_get_cb(SSL *, unsigned char *, int, int *);
+NOEXPORT SSL_SESSION *sess_get_cb(SSL *,
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
+    const
+#endif
+    unsigned char *, int, int *);
 NOEXPORT void sess_remove_cb(SSL_CTX *, SSL_SESSION *);
 
 /* sessiond interface */
 NOEXPORT void cache_new(SSL *, SSL_SESSION *);
-NOEXPORT SSL_SESSION *cache_get(SSL *, unsigned char *, int);
+NOEXPORT SSL_SESSION *cache_get(SSL *, const unsigned char *, int);
 NOEXPORT void cache_remove(SSL_CTX *, SSL_SESSION *);
 NOEXPORT void cache_transfer(SSL_CTX *, const u_char, const long,
     const u_char *, const size_t,
@@ -101,10 +108,10 @@ NOEXPORT void sslerror_log(unsigned long, char *);
 
 /**************************************** initialize section->ctx */
 
-#if OPENSSL_VERSION_NUMBER<0x10100000L
-typedef long SSL_OPTIONS_TYPE;
-#else /* OpenSSL >= 1.1.0 */
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
 typedef long unsigned SSL_OPTIONS_TYPE;
+#else
+typedef long SSL_OPTIONS_TYPE;
 #endif
 
 int context_init(SERVICE_OPTIONS *section) { /* init SSL context */
@@ -256,7 +263,7 @@ NOEXPORT int servername_cb(SSL *ssl, int *ad, void *arg) {
             return SSL_TLSEXT_ERR_OK;
         }
     s_log(LOG_ERR, "SNI: no pattern matched servername: %s", servername);
-    return SSL_TLSEXT_ERR_ALERT_FATAL;
+    return SSL_TLSEXT_ERR_OK;
 }
 /* TLSEXT callback return codes:
  *  - SSL_TLSEXT_ERR_OK
@@ -298,9 +305,9 @@ NOEXPORT int dh_init(SERVICE_OPTIONS *section) {
         DH_free(dh);
         return 0; /* OK */
     }
-    CRYPTO_r_lock(stunnel_locks[LOCK_DH]);
+    CRYPTO_THREAD_read_lock(stunnel_locks[LOCK_DH]);
     SSL_CTX_set_tmp_dh(section->ctx, dh_params);
-    CRYPTO_r_unlock(stunnel_locks[LOCK_DH]);
+    CRYPTO_THREAD_read_unlock(stunnel_locks[LOCK_DH]);
     dh_needed=1; /* generate temporary DH parameters in cron */
     section->option.dh_needed=1; /* update this context */
     s_log(LOG_INFO, "Using dynamic DH parameters");
@@ -447,6 +454,12 @@ NOEXPORT int auth_init(SERVICE_OPTIONS *section) {
         key_needed=load_key_engine(section);
     }
 #endif
+    if (cert_needed && pkcs12_extension(section->cert)) {
+        if (load_pkcs12_file(section)) {
+            return 1; /* FAILED */
+        }
+        cert_needed=key_needed=0; /* don't load any PEM files */
+    }
     if(cert_needed && load_cert_file(section))
         return 1; /* FAILED */
     if(key_needed && load_key_file(section))
@@ -569,6 +582,79 @@ PSK_KEYS *psk_find(const PSK_TABLE *table, const char *identity) {
 
 #endif /* !defined(OPENSSL_NO_PSK) */
 
+NOEXPORT int pkcs12_extension(const char *filename) {
+    const char *ext=strrchr(filename, '.');
+    return ext && (!strcasecmp(ext, ".p12") || !strcasecmp(ext, ".pfx"));
+}
+
+NOEXPORT int load_pkcs12_file(SERVICE_OPTIONS *section) {
+    int i, success;
+    UI_DATA ui_data;
+    BIO *bio=NULL;
+    PKCS12 *p12=NULL;
+    X509 *cert=NULL;
+    STACK_OF(X509) *ca=NULL;
+    EVP_PKEY *pkey=NULL;
+    char pass[PEM_BUFSIZE];
+
+    s_log(LOG_INFO, "Loading certificate and private key from file: %s",
+        section->cert);
+    if(file_permissions(section->cert))
+        return 1; /* FAILED */
+
+    bio=BIO_new_file(section->cert, "rb");
+    if(!bio) {
+        sslerror("BIO_new_file");
+        return 1; /* FAILED */
+    }
+    p12=d2i_PKCS12_bio(bio, NULL);
+    if(!p12) {
+        sslerror("d2i_PKCS12_bio");
+        BIO_free(bio);
+        return 1; /* FAILED */
+    }
+    BIO_free(bio);
+
+    ui_data.section=section; /* setup current section for callbacks */
+
+    /* try the cached value (initially an empty passphrase) */
+    passphrase_cb(pass, PEM_BUFSIZE, 0, NULL);
+    success=PKCS12_parse(p12, pass, &pkey, &cert, &ca);
+
+    /* invoke the UI */
+    for(i=0; !success && i<3; i++) {
+        if(!ui_retry())
+            break;
+        if(i==0) { /* silence the cached attempt */
+            ERR_clear_error();
+        } else {
+            sslerror_queue(); /* dump the error queue */
+            s_log(LOG_ERR, "Wrong passphrase: retrying");
+        }
+        passphrase_cb(pass, PEM_BUFSIZE, 0, &ui_data);
+        success=PKCS12_parse(p12, pass, &pkey, &cert, &ca);
+    }
+    if(!success) {
+        sslerror("PKCS12_parse");
+        PKCS12_free(p12);
+        return 1; /* FAILED */
+    }
+
+    PKCS12_free(p12);
+
+    if(!SSL_CTX_use_certificate(section->ctx, cert)) {
+        sslerror("SSL_CTX_use_certificate");
+        return 1; /* FAILED */
+    }
+    if(!SSL_CTX_use_PrivateKey(section->ctx, pkey)) {
+        sslerror("SSL_CTX_use_PrivateKey");
+        return 1; /* FAILED */
+    }
+    s_log(LOG_INFO, "Certificate and private key loaded from file: %s",
+        section->cert);
+    return 0; /* OK */
+}
+
 NOEXPORT int load_cert_file(SERVICE_OPTIONS *section) {
     s_log(LOG_INFO, "Loading certificate from file: %s", section->cert);
     if(!SSL_CTX_use_certificate_chain_file(section->ctx, section->cert)) {
@@ -579,10 +665,8 @@ NOEXPORT int load_cert_file(SERVICE_OPTIONS *section) {
     return 0; /* OK */
 }
 
-static int cache_initialized=0;
-
 NOEXPORT int load_key_file(SERVICE_OPTIONS *section) {
-    int i, reason;
+    int i, success;
     UI_DATA ui_data;
 
     s_log(LOG_INFO, "Loading private key from file: %s", section->key);
@@ -590,22 +674,28 @@ NOEXPORT int load_key_file(SERVICE_OPTIONS *section) {
         return 1; /* FAILED */
 
     ui_data.section=section; /* setup current section for callbacks */
-    SSL_CTX_set_default_passwd_cb(section->ctx, password_cb);
+    SSL_CTX_set_default_passwd_cb(section->ctx, passphrase_cb);
 
-    for(i=0; i<=3; i++) {
-        if(!i && !cache_initialized)
-            continue; /* there is no cached value */
-        SSL_CTX_set_default_passwd_cb_userdata(section->ctx,
-            i ? &ui_data : NULL); /* try the cached password first */
-        if(SSL_CTX_use_PrivateKey_file(section->ctx, section->key,
-                SSL_FILETYPE_PEM))
+    /* try the cached value (initially an empty passphrase) */
+    SSL_CTX_set_default_passwd_cb_userdata(section->ctx, NULL);
+    success=SSL_CTX_use_PrivateKey_file(section->ctx, section->key,
+        SSL_FILETYPE_PEM);
+
+    /* invoke the UI */
+    SSL_CTX_set_default_passwd_cb_userdata(section->ctx, &ui_data);
+    for(i=0; !success && i<3; i++) {
+        if(!ui_retry())
             break;
-        reason=ERR_GET_REASON(ERR_peek_error());
-        if(i<=2 && reason==EVP_R_BAD_DECRYPT) {
+        if(i==0) { /* silence the cached attempt */
+            ERR_clear_error();
+        } else {
             sslerror_queue(); /* dump the error queue */
-            s_log(LOG_ERR, "Wrong pass phrase: retrying");
-            continue;
+            s_log(LOG_ERR, "Wrong passphrase: retrying");
         }
+        success=SSL_CTX_use_PrivateKey_file(section->ctx, section->key,
+            SSL_FILETYPE_PEM);
+    }
+    if(!success) {
         sslerror("SSL_CTX_use_PrivateKey_file");
         return 1; /* FAILED */
     }
@@ -638,7 +728,7 @@ NOEXPORT int load_cert_engine(SERVICE_OPTIONS *section) {
 }
 
 NOEXPORT int load_key_engine(SERVICE_OPTIONS *section) {
-    int i, reason;
+    int i;
     UI_DATA ui_data;
     EVP_PKEY *pkey;
     UI_METHOD *ui_method;
@@ -646,22 +736,16 @@ NOEXPORT int load_key_engine(SERVICE_OPTIONS *section) {
     s_log(LOG_INFO, "Initializing private key on engine ID: %s", section->key);
 
     ui_data.section=section; /* setup current section for callbacks */
-    SSL_CTX_set_default_passwd_cb(section->ctx, password_cb);
+    SSL_CTX_set_default_passwd_cb(section->ctx, passphrase_cb);
 
-#ifdef USE_WIN32
-    ui_method=UI_create_method("stunnel WIN32 UI");
-    UI_method_set_reader(ui_method, pin_cb);
-#else /* USE_WIN32 */
-    ui_method=UI_OpenSSL();
+    ui_method=UI_stunnel();
     /* workaround for broken engines */
     /* ui_data.section=NULL; */
-#endif /* USE_WIN32 */
-    for(i=1; i<=3; i++) {
+    for(i=0; i<3; i++) {
         pkey=ENGINE_load_private_key(section->engine, section->key,
             ui_method, &ui_data);
         if(!pkey) {
-            reason=ERR_GET_REASON(ERR_peek_error());
-            if(i<=2 && (reason==7 || reason==160)) { /* wrong PIN */
+            if(i<2 && ui_retry()) { /* wrong PIN */
                 sslerror_queue(); /* dump the error queue */
                 s_log(LOG_ERR, "Wrong PIN: retrying");
                 continue;
@@ -680,29 +764,72 @@ NOEXPORT int load_key_engine(SERVICE_OPTIONS *section) {
 
 #endif /* !defined(OPENSSL_NO_ENGINE) */
 
-NOEXPORT int password_cb(char *buf, int size, int rwflag, void *userdata) {
-    static char cache[PEM_BUFSIZE];
+NOEXPORT int passphrase_cb(char *buf, int size, int rwflag, void *userdata) {
+    static char cache[PEM_BUFSIZE]=""; /* try an empty passphrase first */
     int len;
 
     if(size>PEM_BUFSIZE)
         size=PEM_BUFSIZE;
 
-    if(userdata) { /* prompt the user */
-#ifdef USE_WIN32
-        len=passwd_cb(buf, size, rwflag, userdata);
-#else
-        len=PEM_def_callback(buf, size, rwflag, NULL);
-#endif
-        memcpy(cache, buf, (size_t)size); /* save in cache */
-        cache_initialized=1;
-    } else { /* try the cached value */
+    if(!userdata) { /* try the cached value first */
         strncpy(buf, cache, (size_t)size);
         buf[size-1]='\0';
         len=(int)strlen(buf);
+    } else { /* prompt the user on subsequent requests */
+        len=passwd_cb(buf, size, rwflag, userdata); /* invoke the UI */
+        memcpy(cache, buf, (size_t)size); /* save in cache */
     }
     return len;
 }
 
+NOEXPORT int ui_retry() {
+    unsigned long err=ERR_peek_error();
+
+    switch(ERR_GET_LIB(err)) {
+    case ERR_LIB_ASN1:
+        return 1;
+    case ERR_LIB_PKCS12:
+        switch(ERR_GET_REASON(err)) {
+        case PKCS12_R_MAC_VERIFY_FAILURE:
+            return 1;
+        default:
+            return 0;
+        }
+    case ERR_LIB_EVP:
+        switch(ERR_GET_REASON(err)) {
+        case EVP_R_BAD_DECRYPT:
+            return 1;
+        default:
+            return 0;
+        }
+    case ERR_LIB_PEM:
+        switch(ERR_GET_REASON(err)) {
+        case PEM_R_BAD_PASSWORD_READ:
+            return 1;
+        default:
+            return 0;
+        }
+    case ERR_LIB_UI:
+        switch(ERR_GET_REASON(err)) {
+        case UI_R_RESULT_TOO_LARGE:
+        case UI_R_RESULT_TOO_SMALL:
+            return 1;
+        default:
+            return 0;
+        }
+    case ERR_LIB_USER: /* PKCS#11 hacks */
+        switch(ERR_GET_REASON(err)) {
+        case 7UL: /* CKR_ARGUMENTS_BAD */
+        case 0xa0UL: /* CKR_PIN_INCORRECT */
+            return 1;
+        default:
+            return 0;
+        }
+    default:
+        return 0;
+    }
+}
+
 /**************************************** session callbacks */
 
 NOEXPORT int sess_new_cb(SSL *ssl, SSL_SESSION *sess) {
@@ -716,6 +843,9 @@ NOEXPORT int sess_new_cb(SSL *ssl, SSL_SESSION *sess) {
 }
 
 NOEXPORT SSL_SESSION *sess_get_cb(SSL *ssl,
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
+        const
+#endif
         unsigned char *key, int key_len, int *do_copy) {
     CLI *c;
 
@@ -734,6 +864,7 @@ NOEXPORT void sess_remove_cb(SSL_CTX *ctx, SSL_SESSION *sess) {
     opt=SSL_CTX_get_ex_data(ctx, index_opt);
     if(opt->option.sessiond)
         cache_remove(ctx, sess);
+    SSL_SESSION_free(sess);
 }
 
 /**************************************** sessiond functionality */
@@ -767,7 +898,7 @@ NOEXPORT void cache_new(SSL *ssl, SSL_SESSION *sess) {
 }
 
 NOEXPORT SSL_SESSION *cache_get(SSL *ssl,
-        unsigned char *key, int key_len) {
+        const unsigned char *key, int key_len) {
     unsigned char *val, *val_tmp=NULL;
     ssize_t val_len=0;
     SSL_SESSION *sess;
@@ -936,6 +1067,28 @@ NOEXPORT void info_callback(const SSL *ssl, int where, int ret) {
 
     c=SSL_get_ex_data((SSL *)ssl, index_cli);
     if(c) {
+        int state=SSL_get_state((SSL *)ssl);
+
+#if 0
+        s_log(LOG_DEBUG, "state = %x", state);
+#endif
+
+        /* log the client certificate request (if received) */
+#ifndef SSL3_ST_CR_CERT_REQ_A
+        if(state==TLS_ST_CR_CERT_REQ)
+#else
+        if(state==SSL3_ST_CR_CERT_REQ_A)
+#endif
+            print_client_CA_list(SSL_get_client_CA_list(ssl));
+#ifndef SSL3_ST_CR_SRVR_DONE_A
+        if(state==TLS_ST_CR_SRVR_DONE)
+#else
+        if(state==SSL3_ST_CR_SRVR_DONE_A)
+#endif
+            if(!SSL_get_client_CA_list(ssl))
+                s_log(LOG_INFO, "Client certificate not requested");
+
+        /* prevent renegotiation DoS attack */
         if((where&SSL_CB_HANDSHAKE_DONE)
                 && c->reneg_state==RENEG_INIT) {
             /* first (initial) handshake was completed, remember this,
@@ -943,8 +1096,6 @@ NOEXPORT void info_callback(const SSL *ssl, int where, int ret) {
             c->reneg_state=RENEG_ESTABLISHED;
         } else if((where&SSL_CB_ACCEPT_LOOP)
                 && c->reneg_state==RENEG_ESTABLISHED) {
-            int state=SSL_get_state((SSL *)ssl);
-
 #ifndef SSL3_ST_SR_CLNT_HELLO_A
             if(state==TLS_ST_SR_CLNT_HELLO
                     || state==TLS_ST_SR_CLNT_HELLO) {
@@ -957,6 +1108,7 @@ NOEXPORT void info_callback(const SSL *ssl, int where, int ret) {
                 c->reneg_state=RENEG_DETECTED;
             }
         }
+
         if(c->opt->log_level<LOG_DEBUG) /* performance optimization */
             return;
     }
